package com.zanxiang.sdk.service.Impl;

import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.SpringUtils;
import com.zanxiang.mybatis.entity.Game;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.params.ProductPayParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.enums.PayTypeEnum;
import com.zanxiang.sdk.service.GamePayWayService;
import com.zanxiang.sdk.service.GameService;
import com.zanxiang.sdk.service.OrderPayService;
import com.zanxiang.sdk.service.PlatformOrderService;
import com.zanxiang.sdk.service.pay.PayBaseService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-09-28
 * @description : 订单支付
 */
@Slf4j
@Service
public class OrderPayServiceImpl implements OrderPayService {

    @Autowired
    public PlatformOrderService platformOrderService;

    @Autowired
    public GamePayWayService gamePayWayService;

    @Autowired
    private GameService gameService;

    @Override
    public ResultMap payCreate(ProductPayParam product, UserData userData) {
        Long userId = userData.getUserId();
        Game game = gameService.getById(userData.getGameId());
        if (game == null) {
            throw new BaseException("参数错误, 游戏信息不存在");
        }
        //创建订单
        PlatformOrderBO orderBO = new PlatformOrderBO();
        orderBO.setAmount(product.getAmount());
        orderBO.setProductName(product.getProductName());
        orderBO.setProductId(product.getProductId());
        orderBO.setMgUserId(userData.getUserId());
        orderBO.setUserId(userData.getUserId());
        orderBO.setGameId(userData.getGameId());
        orderBO.setCpId(game.getCpId());
        orderBO.setCpOrderId(product.getOrderId());
        orderBO.setFromDevice(userData.getDeviceSystem());
        orderBO.setPayDevice(userData.getDeviceType());
        String orderId = platformOrderService.create(orderBO);
        product.setOrderId(orderId);
        //创建支付参数
        ProductPayParamBO bo = new ProductPayParamBO();
        bo.setGameId(userData.getGameId().toString());
        bo.setUserId(userId);
        bo.setPayDevice(product.getPayDevice() == null ? userData.getDeviceType() : product.getPayDevice());
        bo.setDeviceSystem(userData.getDeviceSystem());
        bo.setSpbillCreateIp(userData.getIp());
        bo.setOutTradeNo(product.getOrderId());
        bo.setPayWay(product.getPayWay());
        bo.setCode(product.getCode());
        //调起支付
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(bo.getPayWay());
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.payCreate(bo);
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response, Integer payType) throws IOException {
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(payType);
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.notify(request, response);
    }

    @Override
    public ResultMap synNotify(HttpServletRequest request, Integer payType) {
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(payType);
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.synNotify(request);
    }

    @Override
    public Boolean payResult(String orderId) {
        Order order = platformOrderService.getById(orderId);
        if (order == null) {
            throw new BaseException("参数错误, 订单信息不存在");
        }
        if (Objects.equals(order.getStatus(), OrderStateEnum.SUCCESS.getCode())) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }
}
